         IMPLICIT REAL(A - H, K - Z), INTEGER(I, J)
         AVER = 1.7
 100     CALL LOGO3(I, AVER)
         IF (I .EQ. 1) CALL ED
         IF (I .EQ. 2) CALL METRIC
         IF (I .EQ. 3) CALL MAP_CRYSTAL_DSPACE
         IF (I .EQ. 4) CALL AANG
         IF (I .EQ. 5) CALL CONV
         IF (I .EQ. 6) CALL MAP_CRYSTAL_PAIR
         IF (I. EQ. 7) CALL MAP_CRYSTAL_CORD
         IF (I. EQ. 8) CALL DOC
         IF (I .EQ. 9) CALL NOTAT
         IF (I .EQ. 10) CALL MAP_CRYSTAL_VECMAG
         IF (I .EQ. 11) GOTO 200
         GOTO 100
  200    STOP
         END

C To convert the components of a vector from real to reciprocal
C   space and vice--versa
C This subroutine works for any crystal system.
C 
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  The metric tensor is explained on page 22
C
      SUBROUTINE MAP_CRYSTAL_CONVERT(F,G,H,K,L,JR1,U,V,W)
      IMPLICIT NONE
C
C JR1 is 0 if vector defined in real space, 1 if in reciprocal space
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
      REAL G(9),F(9),H,K,L,U,V,W
      INTEGER JR1
C
      IF(JR1 .EQ. 0) GOTO 10
         CALL MAP_UTIL_TRANS(F,H,K,L,U,V,W)
      GOTO 20
 10      CALL MAP_UTIL_TRANS(G,H,K,L,U,V,W)
C
 20   RETURN
      END


C To multiply a 3x3 matrix with a single column matrix
C

C To calculate the coordinate transformation matrix relating 
C   two crystals of arbitrary structure, from data consisting
C   of a pair of vectors (real or reciprocal) from each crystal.
C   The pair from one crystal has to be parallel to that from 
C   the other crystal. This is useful in studying the orientation
C   relationships between crystals.
C
C This subroutine works for any crystal system.
C 
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  See pages 12-18 including Examples 4 and 5.
C

      SUBROUTINE MAP_CRYSTAL_CORD(H1,K1,L1,JR1,H2,K2,L2,JR2,H4,K4,L4,
     &JR4,H5,K5,L5,JR5,C,D,G1,G2)
      IMPLICIT NONE
      REAL G1(9),F1(9),G2(9),F2(9),A(9),B(9),C(9),D(9)
      REAL H1,K1,L1,H2,K2,L2,H3,K3,L3,H4,K4,L4,H5,K5,L5,H6,K6,L6
      REAL M1,M2,M3,M4,M5,M6,KP,GP,MP
      INTEGER JR1,JR2,JR3,JR4,JR5,JR6
C
C F1 and F2 are described as F, and G1 and G2 as G in what follows.
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C 
      CALL MAP_UTIL_INVERS(F1,G1)
      CALL MAP_UTIL_INVERS(F2,G2)
C
C Find vector [H3 K3 L3] which is normal to [H1 K1 L1] and [H2 K2 L2]
      CALL MAP_UTIL_CROSS(H1,K1,L1,JR1,H2,K2,L2,JR2,H3,K3,L3,JR3,F1,G1)
C Find vector [H6 K6 L6] which is normal to [H4 K4 L4] and [H5 K5 L5]
      CALL MAP_UTIL_CROSS(H4,K4,L4,JR4,H5,K5,L5,JR5,H6,K6,L6,JR6,F2,G2)
C
C Find the magnitude of each of the six vectors
         CALL MAP_UTIL_MAGG(H1,K1,L1,G1,M1)
         CALL MAP_UTIL_MAGG(H2,K2,L2,G1,M2)
         CALL MAP_UTIL_MAGG(H3,K3,L3,G1,M3)
         CALL MAP_UTIL_MAGG(H4,K4,L4,G2,M4)
         CALL MAP_UTIL_MAGG(H5,K5,L5,G2,M5)
         CALL MAP_UTIL_MAGG(H6,K6,L6,G2,M6)
C
C Hence calculate the normalisation constants
C  see for example, the constants k, g, m on page 17
       KP=M1/M4
       GP=M2/M5
       MP=M3/M6
C
C Calculate the matrices which are manipulated to enable the
C   coordinate transformation matrix D, and its inverse C
C
         A(1)=H1*KP
         A(4)=K1*KP
         A(7)=L1*KP
         A(2)=H2*GP
         A(5)=K2*GP
         A(8)=L2*GP
         A(3)=H3*MP
         A(6)=K3*MP
         A(9)=L3*MP
C 
         B(1)=H4
         B(2)=H5
         B(3)=H6
         B(4)=K4
         B(5)=K5
         B(6)=K6
         B(7)=L4
         B(8)=L5
         B(9)=L6
C
      CALL MAP_UTIL_INVERS(C,B)
      CALL MAP_UTIL_PROD(D,A,C)
C
      CALL MAP_UTIL_INVERS(C,D)
C
C
      RETURN
      END

C Inverts a 3x3 matrix 
C A is the inverse of B
C
      SUBROUTINE MAP_UTIL_INVERS(A,B)
      IMPLICIT NONE
      REAL A(9),B(9),C1
C
      C1=(B(5)*B(9)-B(6)*B(8))*B(1)+(B(6)*B(7)-B(4)*B(9))*B(2)
     &+(B(4)*B(8)-B(5)*B(7))*B(3)
      A(1)=(B(5)*B(9)-B(6)*B(8))/C1
      A(4)=(B(6)*B(7)-B(4)*B(9))/C1
      A(7)=(B(4)*B(8)-B(5)*B(7))/C1
      A(2)=(B(8)*B(3)-B(9)*B(2))/C1
      A(5)=(B(9)*B(1)-B(7)*B(3))/C1
      A(8)=(B(7)*B(2)-B(8)*B(1))/C1
      A(3)=(B(2)*B(6)-B(3)*B(5))/C1
      A(6)=(B(3)*B(4)-B(1)*B(6))/C1
      A(9)=(B(1)*B(5)-B(2)*B(4))/C1
C
        RETURN
        END


C To calculate the magnitude of a lattice vector
C  given the metric tensor G
C
      SUBROUTINE MAP_UTIL_MAGG(H,K,L,G,M)
      IMPLICIT NONE
      REAL H,K,L,M,A,B,C,G(9)
C
        CALL MAP_UTIL_TRANS(G,H,K,L,A,B,C)
          M=H*A+K*B+L*C
          M=SQRT(M)
          M=1.0/M
C
      RETURN
      END
C To evaluate the product A  of two 3x3 matrices B and C
C
      SUBROUTINE MAP_UTIL_PROD(A,B,C)
      IMPLICIT NONE
      REAL A(9),B(9),C(9)
C
        A(1)=B(1)*C(1)+B(2)*C(4)+B(3)*C(7)
        A(2)=B(1)*C(2)+B(2)*C(5)+B(3)*C(8)
        A(3)=B(1)*C(3)+B(2)*C(6)+B(3)*C(9)
        A(4)=B(4)*C(1)+B(5)*C(4)+B(6)*C(7)
        A(5)=B(4)*C(2)+B(5)*C(5)+B(6)*C(8)
        A(6)=B(4)*C(3)+B(5)*C(6)+B(6)*C(9)
        A(7)=B(7)*C(1)+B(8)*C(4)+B(9)*C(7)
        A(8)=B(7)*C(2)+B(8)*C(5)+B(9)*C(8)
        A(9)=B(7)*C(3)+B(8)*C(6)+B(9)*C(9)
C
         RETURN
         END


      SUBROUTINE MAP_UTIL_TRANS(LR,X1,X2,X3,X4,X5,X6)
      IMPLICIT NONE
      REAL LR(9),X1,X2,X3,X4,X5,X6
C
        X4=LR(1)*X1+LR(2)*X2+LR(3)*X3
        X5=LR(4)*X1+LR(5)*X2+LR(6)*X3
        X6=LR(7)*X1+LR(8)*X2+LR(9)*X3
C
      RETURN
      END

C To interactively solve electron diffraction patterns
C   given a pair of reciprocal lattice vectors of unknown
C   indices, and an acute angle between these vectors
C   The d-spacings of these vectors can be input, or just a ratio
C   of their magnitudes.
C
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C ACCU is the measurement accuracy expressed as a fraction
C GV1 and GV2 are the d-spacings corresponding to the two input vectors
C
      SUBROUTINE MAP_CRYSTAL_ED1(F,ACCU,AAA,PI,GV1,GV2,RR,J21,
     &JTYPE,AH,AK,AL,AAD,AHH,AKK,ALL,ABD,AU1,AV1,AW1,AAX,JA)
      IMPLICIT NONE
      REAL F(9),ACCU,AAA,PI,GV1,GV2,RR,AH(400),AK(400),
     &AL(400),AAD(400),AHH(400),AKK(400),ALL(400),ABD(400),AU1(400),
     &AV1(400),AW1(400),AAX(400),AA,H,K,L,HH,KK,LL,AD,BD,U1,V1,W1,AX
      INTEGER J20,J21,J22,JI,JA,I1,I2,I3,JTYPE,J1,J2,J3
C
C
        J22=J21+1
        J20=2*J21+1
        J21=-J21-1
C
      IF(ACCU .GT. 0.08) ACCU=0.08
C
C
      RR=GV1/GV2
C
C
      IF(RR .GE. 1.0) GOTO 8
      RR=1.0/RR
C Convert angle to radians
8     AA=AAA*PI/180.0
C 
C Systematically vary the Miller indices of the pair of input
C  vectors, so that the trial vectors can be tested against
C  those input, with respect to magnitude and angle
C
C
      JI=1
C
C
      DO 1 I1=1,J22
        H=I1-1
          DO 2 I2=1,J20
            K=I2+J21
              DO 3 I3=1,J20
                L=I3+J21
                 DO 4 J1=1,J22
                  HH=J1-1
                    DO 5 J2=1,J20
                      KK=J2+J21
                        DO 6 J3=1,J20
                         LL=J3+J21
C
      IF(H.EQ.0.AND.K.EQ.0.AND.L.EQ.0)GOTO6
      IF(HH.EQ.0.AND.KK.EQ.0.AND.LL.EQ.0)GOTO6
      IF(HH.EQ.H.AND.KK.EQ.K.AND.LL.EQ.L)GOTO6
      IF(-1*HH.EQ.H.AND.-1*KK.EQ.K.AND.-1*LL.EQ.L)GOTO6
C
      CALL MAP_CRYSTAL_TEST1(ACCU,F,GV1,GV2,RR,AA,H,K,L,
     &HH,KK,LL,JTYPE,AD,BD,U1,V1,W1,AX,PI)
C
C
      AH(JI)=H
      AK(JI)=K
      AL(JI)=L
      AAD(JI)=AD
      AHH(JI)=HH
      AKK(JI)=KK
      ALL(JI)=LL
      ABD(JI)=BD
      AU1(JI)=U1
      AV1(JI)=V1
      AW1(JI)=W1
      AAX(JI)=AX
      JI=JI+1

C
6                               CONTINUE
5                             CONTINUE
4                          CONTINUE
3                      CONTINUE
2                  CONTINUE
1               CONTINUE
C
C
      JA=JI
      RETURN
      END

C To interactively solve electron diffraction patterns
C   given a pair of reciprocal lattice vectors of unknown
C   indices, and an acute angle between these vectors
C   The d-spacings of these vectors can be input, or just a ratio
C   of their magnitudes.
C
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C ACCU is the measurement accuracy expressed as a fraction
C GV1 and GV2 are the d-spacings corresponding to the two input vectors
C
      SUBROUTINE MAP_CRYSTAL_ED2(F,ACCU,AAA,PI,RR,J21,JTYPE,
     &AH,AK,AL,AAD,AHH,AKK,ALL,ABD,AU1,AV1,AW1,AAX,JA)
      IMPLICIT NONE
      REAL F(9),ACCU,AAA,PI,RR,AH(400),AK(400),
     &AL(400),AAD(400),AHH(400),AKK(400),ALL(400),ABD(400),AU1(400),
     &AV1(400),AW1(400),AAX(400),AA,H,K,L,HH,KK,LL,U1,V1,W1,AD,BD,AX
      INTEGER J20,J21,J22,JI,JA,I1,I2,I3,JTYPE,J1,J2,J3
C
C
        J22=J21+1
        J20=2*J21+1
        J21=-J21-1
C
      IF(ACCU .GT. 0.08) ACCU=0.08
C
C
C
C
      IF(RR .GE. 1.0) GOTO 8
      RR=1.0/RR
C Convert angle to radians
8     AA=AAA*PI/180.0
C 
C Systematically vary the Miller indices of the pair of input
C  vectors, so that the trial vectors can be tested against
C  those input, with respect to magnitude and angle
C
C
      JI=1
C
C
      DO 1 I1=1,J22
        H=I1-1
          DO 2 I2=1,J20
            K=I2+J21
              DO 3 I3=1,J20
                L=I3+J21
                 DO 4 J1=1,J22
                  HH=J1-1
                    DO 5 J2=1,J20
                      KK=J2+J21
                        DO 6 J3=1,J20
                         LL=J3+J21
C
      IF(H.EQ.0.AND.K.EQ.0.AND.L.EQ.0)GOTO6
      IF(HH.EQ.0.AND.KK.EQ.0.AND.LL.EQ.0)GOTO6
      IF(HH.EQ.H.AND.KK.EQ.K.AND.LL.EQ.L)GOTO6
      IF(-1*HH.EQ.H.AND.-1*KK.EQ.K.AND.-1*LL.EQ.L)GOTO6
C
      CALL MAP_CRYSTAL_TEST2(ACCU,F,RR,AA,H,K,L,
     &HH,KK,LL,JTYPE,AD,BD,U1,V1,W1,AX,PI)
C
C
      AH(JI)=H
      AK(JI)=K
      AL(JI)=L
      AAD(JI)=AD
      AHH(JI)=HH
      AKK(JI)=KK
      ALL(JI)=LL
      ABD(JI)=BD
      AU1(JI)=U1
      AV1(JI)=V1
      AW1(JI)=W1
      AAX(JI)=AX
      JI=JI+1

C
6                               CONTINUE
5                             CONTINUE
4                          CONTINUE
3                      CONTINUE
2                  CONTINUE
1               CONTINUE
C
C
      JA=JI
      RETURN
      END

C To calculate the magnitude of a vector defined in real space
C
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  pages 5,6 and equation 4a-c.
C The metric tensor G converts the components from real to reciprocal
C  space.
C
      SUBROUTINE MAP_CRYSTAL_VECMAG(G,U,V,W,M)
      IMPLICIT NONE
      REAL G(9),U,V,W,M,H,K,L
C
C
C
      CALL MAP_UTIL_TRANS(G,U,V,W,H,K,L)
      M=SQRT(U*H+V*K+W*L)
C
      RETURN
      END
C---------------------------------------------------------------
C To calculate the axis-angle pair relating two cubic lattices
C  from an input which consists of a pair of vectors from one
C  crystal, and another pair from the second crystal, together
C  with an angle between the two sets. All vectors are coplanar.
C All 24 symmetry-related axis-angle pairs are computed.
C
      SUBROUTINE MAP_CRYSTAL_PAIR(H,K,L,H1,K1,L1,U1,V1,W1,U,V,W,
     &BEC,R,AINVR,PI,X,Y,Z,N1,Q1,Q2,Q3,THET,PHI)
      IMPLICIT NONE
      REAL H,K,L,H1,K1,L1,U1,V1,W1,U,V,W,PI,PHI
      REAL H2,K2,L2,Q,M,P,A1,A2,A3,A4,A5,A6,A7,A8,A9
      REAL C2,C3,C4,C5,C,C10,H3,K3,L3,H4,K4,L4
      REAL X,Y,Z,N,N1,XXX,YYY,ZZZ,NNNN1
      REAL BEC(24,9),B(9),AA(9),R(9),AINVR(9)
      REAL Q1(23),Q2(23),Q3(23),THET(23)
      INTEGER I,I20,I21
C
        PHI=PHI*2.0*PI/360.0
C Normalize the components of each input vector, i.e. redefine
C  in an orthonormal coordinates
        CALL MAP_UTIL_NORM(H,K,L)
        CALL MAP_UTIL_NORM(H1,K1,L1)
        CALL MAP_UTIL_NORM(U1,V1,W1)
        CALL MAP_UTIL_NORM(U,V,W)
C For crystal 1, calculate a third orthogonal vector using a vector
C   cross product
        H2=K*L1-L*K1
        K2=L*H1-H*L1
        L2=H*K1-K*H1
        CALL MAP_UTIL_NORM(H2,K2,L2)
C Calculate angle between the pair of input vectors of crystal 1
        Q=H*H1+K*K1+L*L1
8       Q=ACOS(Q)
C Calculate the angle between the pair of input vectors of crystal 2
        M=U*U1+V*V1+W*W1
10      M=ACOS(M)
C Calculate a further angle P
      P=M-PHI
C
        A1=H2
        A2=K2
        A3=L2
        A4=H1
        A5=K1
        A6=L1
        A7=H
        A8=K
        A9=L
C
        C2=COS(PHI)
        C3=COS(PHI+Q)
        C4=COS(P)
        C5=COS(Q-P)
C
      B(1)=A5*A9-A6*A8
      B(2)=A6*A7-A4*A9
      B(3)=A4*A8-A7*A5
      B(4)=A8*A3-A9*A2
      B(5)=A9*A1-A7*A3
      B(6)=A7*A2-A8*A1
      B(7)=A2*A6-A3*A5
      B(8)=A3*A4-A1*A6
      B(9)=A1*A5-A2*A4
      C=A1*B(1)+A2*B(2)+A3*B(3)
C
      DO 11 I=1,9
      AA(I)=B(I)/C
11    CONTINUE
      H3=AA(4)*C2+AA(7)*C3
      K3=AA(5)*C2+AA(8)*C3
      L3=AA(6)*C2+AA(9)*C3
      H4=AA(4)*C4+AA(7)*C5
      K4=AA(5)*C4+AA(8)*C5
      L4=AA(6)*C4+AA(9)*C5
      CALL MAP_UTIL_NORM(H3,K3,L3)
      CALL MAP_UTIL_NORM(H4,K4,L4)
      X=(K4-V1)*(L3-W)-(L4-W1)*(K3-V)
      Y=(L4-W1)*(H3-U)-(H4-U1)*(L3-W)
      Z=(H4-U1)*(K3-V)-(K4-V1)*(H3-U)
      C10=(H4+U1)*(H3-U)+(K4+V1)*(K3-V)+(L3-W)*(L4+W1)
      X=X/C10
      Y=Y/C10
      Z=Z/C10
      M=SQRT(X*X+Y*Y+Z*Z)
      N=2.0*ATAN(M)
      N1=N*360.0/(2.0*PI)
      CALL MAP_UTIL_NORM(X,Y,Z)
C
C
      CALL MAP_UTIL_ROT(X,Y,Z,N,R)
C 
        XXX=X
        YYY=Y
        ZZZ=Z
        NNNN1=N1
C
C Initialize matrix BEC
        DO 450 I20=1,24
          DO 451 I21=1,9
           BEC(I20,I21)=0.0
451       CONTINUE
450     CONTINUE
C Calculate the 24 symmetry related rotation matrices
      CALL MAP_CRYSTAL_ORIENT(BEC)
C      
C
C Calculate the inverse of the rotation matrix
      CALL MAP_UTIL_INVERS(AINVR,R)
      CALL MAP_CRYSTAL_ROTAT(XXX,YYY,ZZZ,NNNN1,BEC,Q1,Q2,Q3,THET)
C
C
        RETURN
        END
C Contains rotation martices defining the symmetry operations of 
C  a cubic lattice
C
      SUBROUTINE MAP_CRYSTAL_ORIENT(BEC)
      IMPLICIT NONE
      REAL BEC(24,9)
      INTEGER I,J
C
      DO 10 J=1,9
      DO 15 I=1,24
      BEC(I,J)=0.0
  15  CONTINUE
  10  CONTINUE
C
C
       BEC(2,2)=1.0
       BEC(2,6)=1.0
       BEC(2,7)=1.0
      BEC(3,3)=1.0
      BEC(3,4)=1.0
      BEC(3,8)=1.0
       BEC(4,1)=-1.0
       BEC(4,6)=-1.0
       BEC(4,8)=-1.0
      BEC(5,2)=-1.0
      BEC(5,4)=-1.0
      BEC(5,9)=-1.0
       BEC(6,3)=-1.0
       BEC(6,5)=-1.0
       BEC(6,7)=-1.0
      BEC(7,1)=1.0
      BEC(7,6)=1.0
      BEC(7,8)=-1.0
       BEC(8,3)=1.0
       BEC(8,5)=1.0
       BEC(8,7)=-1.0
      BEC(9,2)=1.0
      BEC(9,4)=1.0
      BEC(9,9)=-1.0
       BEC(10,3)=-1.0
       BEC(10,4)=-1.0
       BEC(10,8)=1.0
      BEC(11,2)=-1.0
      BEC(11,6)=-1.0
      BEC(11,7)=1.0
       BEC(12,1)=-1.0
       BEC(12,5)=-1.0
       BEC(12,9)=1.0
      BEC(13,1)=1.0
      BEC(13,6)=-1.0
      BEC(13,8)=1.0
       BEC(14,3)=1.0
       BEC(14,5)=-1.0
       BEC(14,7)=1.0
      BEC(15,2)=1.0
      BEC(15,4)=-1.0
      BEC(15,9)=1.0
       BEC(16,2)=-1.0
       BEC(16,6)=1.0
       BEC(16,7)=-1.0
      BEC(17,3)=-1.0
      BEC(17,4)=1.0
      BEC(17,8)=-1.0
       BEC(18,1)=-1.0
       BEC(18,5)=1.0
       BEC(18,9)=-1.0
      BEC(19,3)=-1.0
      BEC(19,5)=1.0
      BEC(19,7)=1.0
       BEC(20,1)=-1.0
       BEC(20,6)=1.0
       BEC(20,8)=1.0
      BEC(21,2)=-1.0
      BEC(21,4)=1.0
      BEC(21,9)=1.0
       BEC(22,2)=1.0
       BEC(22,6)=-1.0
       BEC(22,7)=-1.0
      BEC(23,3)=1.0
      BEC(23,4)=-1.0
      BEC(23,8)=-1.0
       BEC(24,1)=1.0
       BEC(24,5)=-1.0
       BEC(24,9)=-1.0
C
      RETURN
      END
C To calculate the 23 equivalent axis-angle pairs.
C 
C The orientation relation between two cubic crystals can be
C   described in terms of an axis-angle pair [P1,P2,P3] and THETA,
C   or alternatively, using a rotation matrix R.
C   However, cubic crystals have a great deal of symmetry so that
C   there are in general 24 crystallographic descriptions of the
C   same orientation relationship. This subroutine the 23 alternative
C   descriptions from a give axis-angle pair. The axis is described
C   with direction the direction cosines [P1 P2 P3] and a right-handed
C   angle of rotation THETA which is input in degrees.
C 
C The matrix BEC contains 24 matrices (3x3), each describing
C   a rotation matrix which is a symmetry operation on the cubic
C   lattice.
C
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C    see pages 18 to 21, 79 (example 25).
C
      SUBROUTINE MAP_CRYSTAL_ROTAT(P1,P2,P3,THETA,BEC,Q1,Q2,Q3,THET,PI)
      IMPLICIT NONE
      REAL P1,P2,P3,THETA,BEC(24,9),Q1(23),Q2(23),Q3(23),THET(23),A5,
     &SYM(9),ANS(9),R(9),PI
      INTEGER I1,I2
C
C ensure that the components P1 P2 P3 are direction cosines
          CALL MAP_UTIL_NORM(P1,P2,P3)
C Convert THETA into radians
          THETA=THETA*PI/180.0
C
C Calculate the the rotation matrix for the given axis-angle pair
      A5=1.0
      CALL MAP_UTIL_ROT(P1,P2,P3,THETA,R)
C
C Calculate the 23 equivalent descriptions
C
      DO 1 I1=1,23
C
C Set SYM to be one of the 24 symmetry operations
           DO 2 I2=1,9
             SYM(I2)=BEC(I1+1,I2)
2         CONTINUE
C
C Generate the new rotation matrix
      CALL MAP_UTIL_PROD(ANS,SYM,R)
C
C Ensure that the new rotation matrix has columns/rows which are
C   unit vectors
        CALL MAP_UTIL_NORM(ANS(1),ANS(2),ANS(3))
        CALL MAP_UTIL_NORM(ANS(4),ANS(5),ANS(6))
        CALL MAP_UTIL_NORM(ANS(7),ANS(8),ANS(9))
C
C Calculate the new rotation angle and rotation axis
C  The method used here does not work for symmetrical R
      THET(I1)=ANS(1)+ANS(5)+ANS(9)
        IF (THET(I1) .LT. -0.999999)  GOTO 999
          Q1(I1)=ANS(8)-ANS(6)
          Q2(I1)=ANS(3)-ANS(7)
          Q3(I1)=ANS(4)-ANS(2)
      GOTO 998
C
C Next few lines are for a symmetrical R
999       THET(I1)=-1.0
          Q1(I1)=SQRT((ANS(1)+1.0)/2.0)
          Q2(I1)=SQRT((ANS(5)+1.0)/2.0)
          Q3(I1)=SQRT((ANS(9)+1.0)/2.0)
C
998       CALL MAP_UTIL_NORM(Q1(I1),Q2(I1),Q3(I1))
          THET(I1)=(THET(I1)-1.0)/2.0
C Convert THETA back to degrees
          THET(I1)=ACOS(THET(I1))*180.0/PI
C
1     CONTINUE
C
        RETURN
        END
C Subroutine to normalise the components of a vector
C
      SUBROUTINE MAP_UTIL_NORM(X,Y,Z)
      IMPLICIT NONE
      REAL X,Y,Z,C
C
        C=SQRT(X*X+Y*Y+Z*Z)
        X=X/C
        Y=Y/C
        Z=Z/C
C
      RETURN
      END
C Calculates a rotation matrix given an axis of rotation and a 
C   right-handed angle of rotation
C
C [P1 P2 P3] are the components of a unit vector which defines
C   the axis of rotation
C THETA is the angle of rotation in radians
C R is the 3x3 rotation matrix
C
C 
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  Equation 8c on page 20
C
      SUBROUTINE MAP_UTIL_ROT(P1,P2,P3,THETA,R)
      IMPLICIT NONE
      REAL P1,P2,P3,THETA,R(9),A5
C
      A5=1.0
       R(1)=P1*P1*(A5-COS(THETA))+COS(THETA)
       R(2)=P1*P2*(A5-COS(THETA))-P3*SIN(THETA)
       R(3)=P1*P3*(A5-COS(THETA))+P2*SIN(THETA)
       R(4)=P2*P1*(A5-COS(THETA))+P3*SIN(THETA)
       R(5)=P2*P2*(A5-COS(THETA))+COS(THETA)
       R(6)=P2*P3*(A5-COS(THETA))-P1*SIN(THETA)
       R(7)=P3*P1*(A5-COS(THETA))-P2*SIN(THETA)
       R(8)=P3*P2*(A5-COS(THETA))+P1*SIN(THETA)
       R(9)=P3*P3*(A5-COS(THETA))+COS(THETA)
C
          RETURN
          END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR1(F,G,AP)
      REAL F(9),G(9),AP
      INTEGER I
C
C Calculates the metric tensor and its inverse for a
C CUBIC crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
          CALL MAP_CRYSTAL_MET1(G,AP)
          CALL MAP_UTIL_INVERS(F,G)
C
C
       RETURN
       END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR2(F,G,AP,CP)
      REAL F(9),G(9),AP,CP
      INTEGER I
C
C Calculates the metric tensor and its inverse for a
C TETRAGONAL crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
          CALL MAP_CRYSTAL_MET2(G,AP,CP)
          CALL MAP_UTIL_INVERS(F,G)

C
       RETURN
       END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR3(F,G,AP,BP,CP)
      REAL F(9),G(9),AP,BP,CP
      INTEGER I
C
C Calculates the metric tensor and its inverse for an
C ORTHORHOMBIC crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
          CALL MAP_CRYSTAL_MET3(G,AP,BP,CP)
          CALL MAP_UTIL_INVERS(F,G)

C
       RETURN
       END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR4(F,G,AP,CP)
      REAL F(9),G(9),AP,CP
      INTEGER I
C
C Calculates the metric tensor and its inverse for a
C HEXAGONAL or TRICLINIC crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
          CALL MAP_CRYSTAL_MET4(G,AP,CP)
          CALL MAP_UTIL_INVERS(F,G)

C
       RETURN
       END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR5(F,G,AP,BP,CP,BETA,PI,IFAIL)
      REAL F(9),G(9),AP,BP,CP,BETA
      INTEGER IFAIL,I
C
C Calculates the metric tensor and its inverse for a
C MONOCLINIC crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
          IFAIL=0
	  IF (BETA .LT. 0.0 .OR. BETA .GT. 180.0) IFAIL=1
	  IF (IFAIL .EQ. 1) GOTO 40
          CALL MAP_CRYSTAL_MET5(G,AP,BP,CP,BETA,PI,IFAIL)
          CALL MAP_UTIL_INVERS(F,G)

C
  40   RETURN
       END
C 
C
      SUBROUTINE MAP_CRYSTAL_TENSOR6
     &  (F,G,AP,BP,CP,ALPH,BETA,GAMM,PI,IFAIL)
      REAL F(9),G(9),AP,BP,CP,ALPH,BETA,GAMM,PI
      INTEGER IFAIL,I
C
C Calculates the metric tensor and its inverse for an
C TRICLINIC crystal structure.
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
C Adapted from subroutine CRYSTAL of the interactive program MAP_CRYSTAL_PROGRAM.
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
      F(I)=0.0D+00
 20   CONTINUE
C
      IFAIL=0
      IF (ALPH .LT. 0.0 .OR. ALPH .GT. 180.0) IFAIL=1
      IF (BETA .LT. 0.0 .OR. BETA .GT. 180.0) IFAIL=1
      IF (GAMM .LT. 0.0 .OR. GAMM .GT. 180.0) IFAIL=1
      IF (IFAIL .EQ. 1) GOTO 40
          CALL MAP_CRYSTAL_MET6(G,AP,BP,CP,ALPH,BETA,GAMM,PI,IFAIL)
          CALL MAP_UTIL_INVERS(F,G)

C
 40    RETURN
       END
C To calculate the metric tensor for a cubic system
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET1(G,AP)
      REAL G(9),AP
      INTEGER I
C
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
      G(1)=AP*AP
      G(5)=G(1)
      G(9)=G(1)
C
      RETURN
      END
C To calculate the metric tensor for a tetragonal system. 
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET2(G,AP,CP)
      REAL G(9),AP,CP
      INTEGER I
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
      G(1)=AP*AP
      G(5)=G(1)
      G(9)=CP*CP
C
      RETURN
      END
C To calculate the metric tensor for an orthorhombic system.
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET3(G,AP,BP,CP)
      REAL G(9),AP,BP,CP
      INTEGER I
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
      G(1)=AP*AP
      G(5)=BP*BP
      G(9)=CP*CP
C
      RETURN
      END
C To calculate the metric tensor for a hexagonal or triclinic system
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET4(G,AP,CP)
      REAL G(9),AP,CP
      INTEGER I
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
      G(1)=AP*AP
      G(2)=-0.5*G(1)
      G(4)=G(2)
      G(5)=G(1)
      G(9)=CP*CP
C
      RETURN
      END
C To calculate the metric tensor for a monoclinic system.
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET5(G,AP,BP,CP,BETA,PI,IFAIL)
      REAL G(9),AP,BP,CP,BETA,PI
      INTEGER IFAIL
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
          IFAIL=0
	  IF (BETA .LT. 0.0 .OR. BETA .GT. 180.0) IFAIL=1
	  IF (IFAIL .EQ. 1) GOTO 40
      G(1)=AP*AP
      G(3)=AP*CP*COS(BETA*2.0*PI/360.0)
      G(5)=BP*BP
      G(7)=G(3)
      G(9)=CP*CP
C
 40   RETURN
      END
C To calculate the metric tensor for a triclinic system.
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Table 3, page 33.
C
      SUBROUTINE MAP_CRYSTAL_MET6(G,AP,BP,CP,ALPH,BETA,GAMM,PI,IFAIL)
      REAL G(9),AP,BP,CP,ALPH,BETA,GAMM,PI
      INTEGER IFAIL,I
C
      DO 20 I=1,9
      G(I)=0.0D+00
 20   CONTINUE
C
          IFAIL=0
	  IF (ALPH .LT. 0.0 .OR. ALPH .GT. 180.0) IFAIL=1
	  IF (BETA .LT. 0.0 .OR. BETA .GT. 180.0) IFAIL=1
	  IF (GAMM .LT. 0.0 .OR. GAMM .GT. 180.0) IFAIL=1
	  IF (IFAIL .EQ. 1) GOTO 40
      G(1)=AP*AP
      G(2)=AP*BP*COS(GAMM*2.0*PI/360.0)
      G(3)=AP*CP*COS(BETA*2.0*PI/360.0)
      G(4)=G(2)
      G(5)=BP*BP
      G(6)=BP*CP*COS(ALPH*2.0*PI/360.0)
      G(7)=G(3)
      G(8)=G(6)
      G(9)=CP*CP
C
 40   RETURN
      END
C To convert from four to three index notation
C  for the hexagonal system
C
C JR2=0 if vector is in real space, JR=1 if in reciprocal space.
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Appendix 2, page 30.
C----------------------------------------------------------------

      SUBROUTINE NOTAT
      IMPLICIT REAL(A-H,K-Z),INTEGER(I,J)
      CALL LOGO(9,1.0)
21    WRITE(*,2)
2     FORMAT(15X,' Four Index to Three Index Notation (= 0)'//
     &       15X,' Three Index to Four Index Notation (= 1) ?'/)
      CALL REEDI(JR1)
      CALL TRY(2,1.0,JR1)
      IF(JR1 .EQ. 1)GOTO 1
      WRITE(*,3)
3     FORMAT(15X,' Components of Vector ?'/)
      READ(*,*)U,V,T,W
      WRITE(*,4)
4     FORMAT(15X,' REAL (= 0) or RECIPROCAL (= 1) ?'/)
      CALL REEDI(JR2)
      CALL TRY(2,1.0,JR2)
C
      CALL MAP_CRYSTAL_NOTAT1(U,V,T,W,JR2,U1,U2,U3)
C
20     WRITE(*,4444)
      WRITE(*,6)U,V,T,W,U1,U2,U3
      WRITE(*,4444)
6     FORMAT(//' Vector in four index notation  = ',4F8.3//
     &         ' Vector in three index notation = ',3F8.3//)
      GOTO 8
1     WRITE(*,3)
      READ(*,*)U,V,W
      WRITE(*,4)
      CALL REEDI(JR2)
      CALL TRY(2,1.0,JR2)
     
      CALL MAP_CRYSTAL_NOTAT2(U,V,W,JR2,U1,U2,U3,U4)
C

22    WRITE(*,4444)
      WRITE(*,6)U1,U2,U4,U3,U,V,W
      WRITE(*,4444)
      GOTO 8
8     WRITE(*,7)
7     FORMAT(10X,' Continue Analysis ? (0 = No, 1 = Yes)'/)
      CALL REEDI(JYES)
      CALL TRY(2,1.0,JYES)
      IF(JYES .EQ. 1)GOTO 21
4444  FORMAT(/'*********************************************',
     &'***********************************'//)
      RETURN
      END



C

      SUBROUTINE MAP_CRYSTAL_NOTAT1(U,V,T,W,JR2,U1,U2,U3)
      IMPLICIT NONE
      REAL U,V,T,W,U1,U2,U3
      INTEGER JR2
C
      IF(JR2 .EQ. 1)GOTO 5
C
        U1=U-T
        U2=V-T
        U3=W
      GOTO 20
C
5       U1=U
        U2=V
        U3=W
C
20    RETURN        
      END                        
C To convert from three to four index notation
C  for the hexagonal system
C
C JR=0 if vector is in real space, JR=1 if in reciprocal space.
C
C Reference:
C  Chapter on Crystallography, by H. K. D. H. Bhadeshia, 
C   in "Microstructural Characterisation" edited by E. Metcalfe,
C   Institute of Metals, London, 1988, Appendix 2, page 30.
C

      SUBROUTINE MAP_CRYSTAL_NOTAT2(U,V,W,JR2,U1,U2,U3,U4)
      IMPLICIT NONE
      REAL U,V,W,U1,U2,U3,U4
      INTEGER JR2
C
C
      IF(JR2 .EQ. 1)GOTO 10
        U1=(2.0*U-V)/3.0
        U2=(2.0*V-U)/3.0
        U3=W
        U4=-1.0*(U+V)/3.0
      GOTO 20
10      U1=U
        U2=V
        U3=W
        U4=-1.0*(U+V)
C
20      RETURN       
        END
C To find the angle between two vectors.
C
C The vectors can be defined by components in real or reciprocal space.
C This subroutine works for any crystal system.
C 
C Reference
C  Worked Examples in the Geometry of Crystals
C  by H. K. D. H. Bhadeshia
C  Published by the Institute of Materials, London, 1987.
C  The metric tensor is explained on page 22
C  Example 8, page 23 illustrates the actual calculation of angle
C       
      SUBROUTINE MAP_CRYSTAL_ANGLEE(F,G,H,K,L,JR1,HH,KK,LL,JR2,
     &ANG,ADEG,PI)
C
C JR1,JR2 have values of 0 for vectors whose components are defined
C  in real space, and 1 if the components are defined in reciprocal
C  space.
C
C ANG, ANGDEG represent the angle in radians and degrees, respectively
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa
      IMPLICIT NONE
      REAL G(9),F(9),H,K,L,HH,KK,LL,ANG,ADEG,H1,K1,L1,
     &HH1,KK1,LL1,AD,BD,A,B,C,PI,MAP_UTIL_MAG
      INTEGER JR1,JR2
C To calculate the angle, it is necessary for one vector to be
C  defined in real space and the other in reciprocal space. The
C  procedure below first identifies whether real or reciprocal, and
C  then calculates the magnitude of each vector, and then takes
C  a "dot" or scalar product of the two vectors to compute the angle.
C
C Calculate the magnitude of a vector whose components are defined
C  in reciprocal space. Components of vectors in reciprocal space
C  are H, K, L and its corresponding components H1, K1, L1 in real
C  space are obtained by multiplying by the metric tensor F.
C  The square of the magnitudge of the vector is then obtained by
C  H*H1+K*K1+L*L1, the operation carried out by function MAG
C
C
      IF(JR1 .EQ. 0) GOTO 1
        CALL MAP_UTIL_TRANS(F,H,K,L,H1,K1,L1)
        A=MAP_UTIL_MAG(H,K,L,H1,K1,L1)
        A=SQRT(A)
        AD=1.0/A
      GOTO 2
C
1     CALL MAP_UTIL_TRANS(G,H,K,L,H1,K1,L1)
        A=MAP_UTIL_MAG(H,K,L,H1,K1,L1)
        A=SQRT(A)
        AD=1.0/A
C
C
2     IF(JR2 .EQ. 0) GOTO 3
        CALL MAP_UTIL_TRANS(F,HH,KK,LL,HH1,KK1,LL1)
        B=MAP_UTIL_MAG(HH,KK,LL,HH1,KK1,LL1)
        B=SQRT(B)
        BD=1.0/B
C
      GOTO 4
3     CALL MAP_UTIL_TRANS(G,HH,KK,LL,HH1,KK1,LL1)
        B=MAP_UTIL_MAG(HH,KK,LL,HH1,KK1,LL1)
        B=SQRT(B)
        BD=1.0/B
C
C
4     IF(JR1 .EQ. 1 .AND. JR2 .EQ. 0) GOTO 5
      IF(JR1 .EQ. 1 .AND.  JR2 .EQ. 1) GOTO 6
      IF(JR1 .EQ. 0 .AND. JR2 .EQ. 0) GOTO 7
      IF(JR1 .EQ. 0 .AND. JR2 .EQ. 1) GOTO 8
C
5     C=MAP_UTIL_MAG(H,K,L,HH,KK,LL)
        GOTO 9
6     C=MAP_UTIL_MAG(H,K,L,HH1,KK1,LL1)
        GOTO 9
7     C=MAP_UTIL_MAG(H1,K1,L1,HH,KK,LL)
        GOTO 9
8     C=MAP_UTIL_MAG(H,K,L,HH,KK,LL)
C
9     ANG=C/(A*B)
C
C Obtain the angle in radians, then convert into degrees
      ADEG=ACOS(ANG)
      ADEG=ADEG*180.0/PI
C
      RETURN
      END
C Function which calculates the scalar product of two 3-d vectors.
C
      REAL FUNCTION MAP_UTIL_MAG(H,K,L,H1,K1,L1)
      IMPLICIT NONE
      REAL H,K,L,H1,K1,L1,MAP_UTIL_MAG
C
      MAP_UTIL_MAG=H*H1+K*K1+L*L1
C
        RETURN
        END
C To obtain the spacing of planes, when the Miller indices are not
C  those which are systematically absent
C
      SUBROUTINE MAP_CRYSTAL_DSP(F,H,K,L,JTYPE,JLOG,AD)
      IMPLICIT NONE
C
C AD is the spacing of the planes with Miller indices (HKL)
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C
      REAL F(9),H,K,L,AD,MAP_UTIL_MAG,H1,K1,L1,A
      INTEGER JTYPE,JLOG
C
C JTYPE represents the lattice type, with 1=primitive etc.
C   For further details on JTYPE see subroutine TYPE
C Note that there are no systematic absences for a primitive lattice
C   so that the calculation can jump straight away to statement 6.
C For all other lattice types, subroutine TYPE determines whether 
C   there is a systematic absence (i.e. JLOG=1) and the spacing is
C   calculated only if there is no absence
      IF(JTYPE .EQ. 1)GOTO 10
        CALL MAP_CRYSTAL_TYPE(H,K,L,JTYPE,JLOG)
      IF(JLOG .EQ. 1)GOTO 20
C Calcuate spacing using a scalar product of the real and reciprocal
C   lattice components of the same vector
 10       CALL MAP_UTIL_TRANS(F,H,K,L,H1,K1,L1)
C
          A=MAP_UTIL_MAG(H,K,L,H1,K1,L1)
          A=SQRT(A)
          AD=1.0/A
C
 20    RETURN
       END
C This helps to determine systematic absences due to lattice type
C
C JTYPE defines the lattice type (face-centered cubic etc.)
C       JTYPE=1 is for primitive
C       JTYPE=2 is for body-centered
C       JTYPE=3 is for face-centered
C       JTYPE=4 is for A-centered
C       JTYPE=5 is for B-centered
C       JTYPE=6 is for C-centered
C Reference: "Essentials of Crystallography"
C            by D. McKie and C. McKie, Blackwell Scientific Publ.
C            London, 1986, page 197
C
C
      SUBROUTINE MAP_CRYSTAL_TYPE(H,K,L,JTYPE,JLOG)
      IMPLICIT NONE
      REAL H,K,L,A
      INTEGER JTYPE,JLOG,JODD,J1,J2,J3
C
C
        IF(JTYPE .EQ. 1)GOTO 2
        IF(JTYPE .EQ. 2)GOTO 3
        IF(JTYPE .EQ. 3)GOTO 4
        IF(JTYPE .EQ. 4)GOTO 5
        IF(JTYPE .EQ. 5)GOTO 6
        IF(JTYPE .EQ. 6)GOTO 8
C
C Now check for systematic absences
C  and set JLOG=0 if the HKL is not absent, 1 if absent
C
3     A=H+K+L
        CALL MAP_UTIL_ODD(A,JODD)
        IF(JODD .EQ. 1)GOTO 7
      GOTO 2
C
4        CALL MAP_UTIL_ODD(H,J1)
         CALL MAP_UTIL_ODD(K,J2)
         CALL MAP_UTIL_ODD(L,J3)
        IF(J1 .EQ. 0 .AND. J2 .EQ. 0 .AND. J3 .EQ. 0) GOTO 2
        IF(J1 .EQ. 1 .AND. J2 .EQ. 1 .AND. J3 .EQ. 1) GOTO 2
      GOTO 7
C
5     A=K+L
        CALL MAP_UTIL_ODD(A,JODD)
        IF(JODD .EQ. 1)GOTO 7
      GOTO 2
C
6     A=H+L
        CALL MAP_UTIL_ODD(A,JODD)
        IF(JODD .EQ. 1) GOTO 7
      GOTO 2
C
8     A=H+K
        CALL MAP_UTIL_ODD(A,JODD)
        IF(JODD .EQ. 1) GOTO 7
      GOTO 2
C
7     JLOG=1
        GOTO 9
2     JLOG=0
C
9       RETURN
        END
       SUBROUTINE MAP_UTIL_ODD(A,JODD)
       IMPLICIT NONE
       REAL A
       INTEGER IH2,IH3,JODD
       IH2=A/2
       IH3=(A+1)/2
       IF (IH2.EQ.IH3) THEN 
       JODD=0
       ELSE
       JODD=1
       ENDIF
       RETURN
       END
C To obtain the spacing of planes, when the Miller indices are not
C  those which are systematically absent. The calculation is
C  interactive, and ordered lists of d-spacings available
C
      SUBROUTINE MAP_CRYSTAL_DSPACE(G,JTYPE,II1,JYES,AH,AK,AL,
     &SPCE,MAGN,JK)
      IMPLICIT NONE
      REAL G(9),F(9),AAA(3000),SPCE(400),MAGN(400)
      REAL AH(400),AK(400),AL(400),H,K,L,AD
      INTEGER JTYPE,II1,JYES,JLOG,JK,I1,I2,I3,IN,II2,JI
C
C
C The metric tensor F converts the components from reciprocal to
C  real space. 
C The metric tensor G converts the components from real to reciprocal
C  space, and G is the inverse of F and vice-versa.
C AAA is the matrix of d-spacings
C
C
C
      JK=0
      II1=II1+1
C
      CALL MAP_UTIL_INVERS(F,G)
C
C Generate a list of spacings with systematically varying Miller indices
      II2=0
      DO 10 I1=1,II1
        H=I1-1
         DO 20 I2=1,II1
          K=I2-1
           DO 30 I3=1,II1
            L=I3-1
            IF(H.EQ.0 .AND. K.EQ.0 .AND. L.EQ.0)GOTO 30
            CALL MAP_CRYSTAL_DSP(F,H,K,L,JTYPE,JLOG,AD)
            IF(JLOG .EQ. 1)GOTO 30
            II2=II2+1
            AAA(II2)=AD
              DO 40  JI=1,II2-1
C              Eliminate planes with similar spacings
               IF(AD .EQ. AAA(JI))GOTO 30
 40           CONTINUE
            IN=INT(H*H+K*K+L*L)
            JK=JK+1
            AH(JK)=H
            AK(JK)=K
            AL(JK)=L
            SPCE(JK)=AD
            MAGN(JK)=IN
 30        CONTINUE
 20      CONTINUE
 10    CONTINUE
C
           IF(JYES .EQ. 0)GOTO 100
C
      CALL MAP_UTIL_SORT2(JK,SPCE,MAGN)
C
C
 100    RETURN
        END
C To calculate the determinant of a 3 x 3 matrix
C
      SUBROUTINE MAP_UTIL_DET(G,DEL)
      IMPLICIT NONE
      REAL G(9),DEL
C
       DEL=G(1)*(G(5)*G(9)-G(6)*G(8))
     &      +G(2)*(G(6)*G(7)-G(4)*G(9))
     &      +G(3)*(G(4)*G(8)-G(7)*G(5))
C
        RETURN
        END
C To multiply a single row matrix with a 3x3 martix
C
      SUBROUTINE MAP_UTIL_TRANS2(D,H,K,L,HH,KK,LL)
      REAL D(9),H,K,L,HH,KK,LL
C
         HH=H*D(1)+K*D(4)+L*D(7)
         KK=H*D(2)+K*D(5)+L*D(8)
         LL=H*D(3)+K*D(6)+L*D(9)
C
      RETURN
      END
